@@ -7,17 +7,17 @@ Replace this with more appropriate tests for your application.
 
 from django.test import TestCase
 
-from bldcontrol.bbcontroller import BitbakeController
+from bldcontrol.bbcontroller import BitbakeController, BuildSetupException
 from bldcontrol.localhostbecontroller import LocalhostBEController
-from bldcontrol.sshbecontroller import SSHBEController
 from bldcontrol.models import BuildEnvironment, BuildRequest
 from bldcontrol.management.commands.runbuilds import Command
 
 import socket
 import subprocess
+import os
 
 # standard poky data hardcoded for testing
-BITBAKE_LAYERS = [type('bitbake_info', (object,), { "giturl": "git://git.yoctoproject.org/poky.git", "dirpath": "", "commit": "HEAD"})]
+BITBAKE_LAYER = type('bitbake_info', (object,), { "giturl": "git://git.yoctoproject.org/poky.git", "dirpath": "", "commit": "HEAD"})
 POKY_LAYERS = [
     type('poky_info', (object,), { "name": "meta", "giturl": "git://git.yoctoproject.org/poky.git", "dirpath": "meta", "commit": "HEAD"}),
     type('poky_info', (object,), { "name": "meta-yocto", "giturl": "git://git.yoctoproject.org/poky.git", "dirpath": "meta-yocto", "commit": "HEAD"}),
@@ -29,6 +29,17 @@ POKY_LAYERS = [
 # we have an abstract test class designed to ensure that the controllers use a single interface
 # specific controller tests only need to override the _getBuildEnvironment() method
 
+test_sourcedir = os.getenv("TTS_SOURCE_DIR")
+test_builddir = os.getenv("TTS_BUILD_DIR")
+test_address = os.getenv("TTS_TEST_ADDRESS", "localhost")
+
+if test_sourcedir == None or test_builddir == None or test_address == None:
+    raise Exception("Please set TTTS_SOURCE_DIR, TTS_BUILD_DIR and TTS_TEST_ADDRESS")
+
+# The bb server will expect a toaster-pre.conf file to exist. If it doesn't exit then we make
+# an empty one here.
+open(test_builddir + 'conf/toaster-pre.conf', 'a').close()
+
 class BEControllerTests(object):
 
     def _serverForceStop(self, bc):
@@ -38,79 +49,66 @@ class BEControllerTests(object):
     def test_serverStartAndStop(self):
         obe =  self._getBuildEnvironment()
         bc = self._getBEController(obe)
-        bc.setLayers(BITBAKE_LAYERS, POKY_LAYERS) # setting layers, skip any layer info
+        try:
+            # setting layers, skip any layer info
+            bc.setLayers(BITBAKE_LAYER, POKY_LAYERS)
+        except NotImplementedError:
+            print "Test skipped due to command not implemented yet"
+            return True
+        # We are ok with the exception as we're handling the git already exists
+        except BuildSetupException:
+            pass
 
-        hostname = self.test_address.split("@")[-1]
+        bc.pokydirname = test_sourcedir
+        bc.islayerset = True
+
+        hostname = test_address.split("@")[-1]
 
         # test start server and stop
-        self.assertTrue(socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect_ex((hostname, 8200)), "Port already occupied")
-        bc.startBBServer("0:0")
-        self.assertFalse(socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect_ex((hostname, 8200)), "Server not answering")
+        bc.startBBServer()
 
-        bc.stopBBServer()
-        self.assertTrue(socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect_ex((hostname, 8200)), "Server not stopped")
+        self.assertFalse(socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect_ex((hostname, int(bc.be.bbport))), "Server not answering")
 
         self._serverForceStop(bc)
 
     def test_getBBController(self):
         obe = self._getBuildEnvironment()
         bc = self._getBEController(obe)
-        bc.setLayers(BITBAKE_LAYERS, POKY_LAYERS) # setting layers, skip any layer info
+        layerSet = False
+        try:
+            # setting layers, skip any layer info
+            layerSet = bc.setLayers(BITBAKE_LAYER, POKY_LAYERS)
+        except NotImplementedError:
+            print "Test skipped due to command not implemented yet"
+            return True
+        # We are ok with the exception as we're handling the git already exists
+        except BuildSetupException:
+            pass
 
-        bbc = bc.getBBController("%d:%d" % (-1, obe.pk))
+        bc.pokydirname = test_sourcedir
+        bc.islayerset = True
+
+        bbc = bc.getBBController()
         self.assertTrue(isinstance(bbc, BitbakeController))
-        bc.stopBBServer()
 
         self._serverForceStop(bc)
 
 class LocalhostBEControllerTests(TestCase, BEControllerTests):
     def __init__(self, *args):
         super(LocalhostBEControllerTests, self).__init__(*args)
-        # hardcoded for Alex's machine; since the localhost BE is machine-dependent,
-        # I found no good way to abstractize this
-        self.test_sourcedir = "/home/ddalex/ssd/yocto"
-        self.test_builddir = "/home/ddalex/ssd/yocto/build"
-        self.test_address = "localhost"
+
 
     def _getBuildEnvironment(self):
         return BuildEnvironment.objects.create(
                 lock = BuildEnvironment.LOCK_FREE,
                 betype = BuildEnvironment.TYPE_LOCAL,
-                address = self.test_address,
-                sourcedir = self.test_sourcedir,
-                builddir = self.test_builddir )
+                address = test_address,
+                sourcedir = test_sourcedir,
+                builddir = test_builddir )
 
     def _getBEController(self, obe):
         return LocalhostBEController(obe)
 
-class SSHBEControllerTests(TestCase, BEControllerTests):
-    def __init__(self, *args):
-        super(SSHBEControllerTests, self).__init__(*args)
-        self.test_address = "ddalex-desktop.local"
-        # hardcoded for ddalex-desktop.local machine; since the localhost BE is machine-dependent,
-        # I found no good way to abstractize this
-        self.test_sourcedir = "/home/ddalex/ssd/yocto"
-        self.test_builddir = "/home/ddalex/ssd/yocto/build"
-
-    def _getBuildEnvironment(self):
-        return BuildEnvironment.objects.create(
-                lock = BuildEnvironment.LOCK_FREE,
-                betype = BuildEnvironment.TYPE_SSH,
-                address = self.test_address,
-                sourcedir = self.test_sourcedir,
-                builddir = self.test_builddir )
-
-    def _getBEController(self, obe):
-        return SSHBEController(obe)
-
-    def test_pathExists(self):
-        obe = BuildEnvironment.objects.create(betype = BuildEnvironment.TYPE_SSH, address= self.test_address)
-        sbc = SSHBEController(obe)
-        self.assertTrue(sbc._pathexists("/"))
-        self.assertFalse(sbc._pathexists("/.deadbeef"))
-        self.assertTrue(sbc._pathexists(sbc._shellcmd("pwd")))
-
-
 class RunBuildsCommandTests(TestCase):
     def test_bec_select(self):
         """
@@ -129,7 +127,7 @@ class RunBuildsCommandTests(TestCase):
         self.assertRaises(IndexError, command._selectBuildEnvironment)
 
     def test_br_select(self):
-        from orm.models import Project, Release, BitbakeVersion
+        from orm.models import Project, Release, BitbakeVersion, Branch
         p = Project.objects.create_project("test", Release.objects.get_or_create(name = "HEAD", bitbake_version = BitbakeVersion.objects.get_or_create(name="HEAD", branch=Branch.objects.get_or_create(name="HEAD"))[0])[0])
         obr = BuildRequest.objects.create(state = BuildRequest.REQ_QUEUED, project = p)
         command = Command()
